"""
CE365 Agent - Internationalization (i18n) System

Copyright (c) 2026 Carsten Eckhardt / Eckhardt-Marketing
Licensed under Source Available License
"""

from .translator import Translator, get_translator, set_language

__all__ = ["Translator", "get_translator", "set_language"]
